@@ -33,6 +33,8 @@ pub struct SynthMinidump {
     unloaded_module_list: Option<ExListStream<UnloadedModule>>,
     /// List of threads in this minidump.
     thread_list: Option<ListStream<Thread>>,
+    /// List of thread names in this minidump.
+    thread_names_list: Option<ListStream<ThreadName>>,
     /// List of memory regions in this minidump.
     memory_list: Option<ListStream<Section>>,
     /// Crashpad extension containing annotations.
@@ -166,6 +168,10 @@ impl SynthMinidump {
                 md::MINIDUMP_STREAM_TYPE::ThreadListStream,
                 endian,
             )),
+            thread_names_list: Some(ListStream::new(
+                md::MINIDUMP_STREAM_TYPE::ThreadNamesStream,
+                endian,
+            )),
             memory_list: Some(ListStream::new(
                 md::MINIDUMP_STREAM_TYPE::MemoryListStream,
                 endian,
@@ -232,6 +238,15 @@ impl SynthMinidump {
         self
     }
 
+    /// Add `thread_name` to `self`, adding it to the thread name stream as well.
+    pub fn add_thread_name(mut self, thread_name: ThreadName) -> SynthMinidump {
+        self.thread_names_list = self
+            .thread_names_list
+            .take()
+            .map(|thread_names_list| thread_names_list.add(thread_name));
+        self
+    }
+
     /// Add crashpad module and annotation extension information.
     pub fn add_crashpad_info(mut self, crashpad_info: CrashpadInfo) -> Self {
         self.crashpad_info = Some(crashpad_info);
@@ -285,6 +300,9 @@ impl SynthMinidump {
         // Add thread list stream if any threads were added.
         let threads = self.thread_list.take();
         self = self.finish_list(threads);
+        // Add thread names stream if any names were added.
+        let thread_names = self.thread_names_list.take();
+        self = self.finish_list(thread_names);
         // Add crashpad info stream if any.
         if let Some(crashpad_info) = self.crashpad_info.take() {
             self = self.add_stream(crashpad_info);
@@ -825,6 +843,32 @@ impl From<Thread> for Section {
     }
 }
 
+/// A minidump thread name.
+pub struct ThreadName {
+    section: Section,
+}
+
+impl ThreadName {
+    pub fn new(endian: Endian, id: u32, name: Option<&DumpString>) -> Self {
+        let section = Section::with_endian(endian).D32(id);
+        // Name is optional to test corrupt handles easily
+        let section = if let Some(name) = name {
+            section.D64(name.file_offset())
+        } else {
+            section.D64(0xFFFF_FFFF_FFFF_FFFF)
+        };
+        ThreadName { section }
+    }
+}
+
+impl_dumpsection!(ThreadName);
+
+impl From<ThreadName> for Section {
+    fn from(thread: ThreadName) -> Self {
+        thread.section
+    }
+}
+
 /// A range of memory contents.
 pub struct Memory {
     section: Section,
@@ -1025,8 +1069,8 @@ impl From<MiscStream> for Section {
         // for that version, leaning on Default to fill in values that are None.
         let mut section = section.D32(flags.bits() as u64 as u32);
 
-        let process_id = process_id.unwrap_or_else(Default::default);
-        let process_times = process_times.unwrap_or_else(Default::default);
+        let process_id = process_id.unwrap_or_default();
+        let process_times = process_times.unwrap_or_default();
         section = section.D32(process_id);
         section = section
             .D32(process_times.process_create_time)
@@ -1034,7 +1078,7 @@ impl From<MiscStream> for Section {
             .D32(process_times.process_kernel_time);
 
         if misc_info_version >= 2 {
-            let power_info = power_info.unwrap_or_else(Default::default);
+            let power_info = power_info.unwrap_or_default();
             section = section
                 .D32(power_info.processor_max_mhz)
                 .D32(power_info.processor_current_mhz)
@@ -1044,10 +1088,10 @@ impl From<MiscStream> for Section {
         }
 
         if misc_info_version >= 3 {
-            let process_integrity_level = process_integrity_level.unwrap_or_else(Default::default);
-            let process_execute_flags = process_execute_flags.unwrap_or_else(Default::default);
-            let protected_process = protected_process.unwrap_or_else(Default::default);
-            let time_zone = time_zone.unwrap_or_else(Default::default);
+            let process_integrity_level = process_integrity_level.unwrap_or_default();
+            let process_execute_flags = process_execute_flags.unwrap_or_default();
+            let protected_process = protected_process.unwrap_or_default();
+            let time_zone = time_zone.unwrap_or_default();
 
             section = section.D32(process_integrity_level);
             section = section.D32(process_execute_flags);
@@ -1081,7 +1125,7 @@ impl From<MiscStream> for Section {
         }
 
         if misc_info_version >= 4 {
-            let build_strings = build_strings.unwrap_or_else(Default::default);
+            let build_strings = build_strings.unwrap_or_default();
             for &val in &build_strings.build_string {
                 section = section.D16(val);
             }
@@ -1091,8 +1135,8 @@ impl From<MiscStream> for Section {
         }
 
         if misc_info_version >= 5 {
-            let misc_5 = misc_5.unwrap_or_else(Default::default);
-            let process_cookie = misc_5.process_cookie.unwrap_or_else(Default::default);
+            let misc_5 = misc_5.unwrap_or_default();
+            let process_cookie = misc_5.process_cookie.unwrap_or_default();
             let xstate = misc_5.xstate_data;
             section = section
                 .D32(xstate.size_of_info)
